@@ -27,6 +27,7 @@
 
 #include "internal.h"
 #include "dbus-maybe.h"
+#include "glibcompat.h"
 #include "notify.h"
 
 static PurpleNotifyUiOps *notify_ui_ops = NULL;
@@ -262,8 +263,7 @@ purple_notify_searchresults_free(PurpleNotifySearchResults *results)
 
 	for (l = results->rows; l; l = g_list_delete_link(l, l)) {
 		GList *row = l->data;
-		g_list_foreach(row, (GFunc)g_free, NULL);
-		g_list_free(row);
+		g_list_free_full(row, (GDestroyNotify)g_free);
 	}
 
 	for (l = results->columns; l; l = g_list_delete_link(l, l)) {
